package go_swagger

import (
	"path/filepath"

	"github.com/juju/errors"

	"git.home.m-and-m.ovh/mderasse/gouick/helpers"
	"git.home.m-and-m.ovh/mderasse/gouick/models"

	log "github.com/sirupsen/logrus"
)

type makefileTemplate struct {
	APIs    []*models.Process
	Crons   []*models.Process
	Workers []*models.Process
}

// GenerateMakefile will generate makefile based on the given config.
// Launched only at project init or on force.
func (a APIType) GenerateMakefile(path string, config *models.Config) error {
	log.Debugf("Starting %s - %s", a.GetName(), helpers.GetCurrentFuncName())

	templatePath := filepath.Join(templateDirectory, "Makefile.tmpl")
	savePath := filepath.Join(path, "Makefile")

	data := makefileTemplate{}

	processes, err := a.getProcesses(path)
	if err != nil {
		log.Error("Fail to list processes")
		return errors.Trace(err)
	}

	for _, process := range processes {
		switch process.Type {
		case models.ProcessType_API:
			data.APIs = append(data.APIs, process)
		case models.ProcessType_CRON:
			data.Crons = append(data.Crons, process)
		case models.ProcessType_WORKER:
			data.Workers = append(data.Workers, process)
		}
	}

	err = helpers.WriteTemplate(templatePath, savePath, data)
	if err != nil {
		return errors.Trace(err)
	}

	return nil
}
